#include <jni.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "jargon_Cle2000.h"
#include "cle2000.h"

/*--------------------------------*/
/* Java-Dragon (Jargon) binding   */
/* author: A. Hebert (17/08/2010) */
/*--------------------------------*/

/*
Copyright (C) 2010 Ecole Polytechnique de Montreal

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
*/

/* exception treatment. Note: These native method must be called from inside
  a try bloc in the Java side */
static char nomstatic[132];
#include "except.h"         /* (try,catch,throw) macros for exception handling */
jmp_buf exception_buffer;   /* environment buffer used by except.h             */
static int exception_status=0;  /* variable holding the error code (except.h)      */

/* Java Dragon binding */
JNIEXPORT void JNICALL Java_jargon_Cle2000_execNat
   (JNIEnv *env, jobject this, jstring filenm, jint iprint, jobject jstack) {
   FILE *filein;
   const char *file_name;
   int_32 dramod(char *cmodul, int_32 nentry, char (*hentry)[13], int_32 *ientry,
          int_32 *jentry, lcm **kentry, char (*hparam)[73]);
   int_32 ipos;
   jlong lifo_stack;
   jclass stackcls;
   jfieldID fid;
   int_32 ier, ilevel = 1;
   if (jstack == NULL) {
     sprintf(nomstatic,"Cle2000Nat: jstack undefined. Use setLifo()");
     jraise_exc(env, nomstatic);
     abort();
   }
   file_name = (*env)->GetStringUTFChars(env, filenm, 0);
   stackcls = (*env)->GetObjectClass(env, jstack);
   fid = (*env)->GetFieldID(env, stackcls, "addr", "J");
   if (fid == NULL) {
     sprintf(nomstatic,"Cle2000Nat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_stack = (*env)->GetLongField(env, jstack, fid);

/* close the LCM objects */
   for (ipos=0; ipos<((lifo*)(long)lifo_stack)->nup; ++ipos) {
     lifo_node *my_node = clepos((lifo**)&lifo_stack, ipos);
     if ((my_node->type == 3) || (my_node->type == 4)) {
       if (my_node->access > 0) {
         try{
           lcmcl_c(&(my_node->value.mylcm), 1);
         } catch(JargonError){
           jraise_exc(env, nomstatic);
           fflush(stdout);
           abort();
         }
       }
     }
   }

/* call the parametrized procedure */
   try{
     ier = cle2000_c(ilevel, &dramod, (char*)file_name, iprint, (lifo*)(long)lifo_stack);
   } catch(JargonError){
     jraise_exc(env, nomstatic);
     abort();
   }
   if (ier != 0) {
     sprintf(nomstatic,"Cle2000Nat: cle2000 failure (%s.c2m). ier=%d", file_name, ier);
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, filenm, file_name); /* Release the character string */

/* reopen the LCM objects */
   for (ipos=0; ipos<((lifo*)(long)lifo_stack)->nup; ++ipos) {
     lifo_node *my_node = clepos((lifo**)&lifo_stack, ipos);
     if ((my_node->type == 3) || (my_node->type == 4)) {
       int access = my_node->access;
       if (my_node->access == 0) access = 1;
       try{
         lcmop_c(&(my_node->value.mylcm), my_node->OSname, access, my_node->type-2, 0);
       } catch(JargonError){
         jraise_exc(env, nomstatic);
         fflush(stdout);
         abort();
       }
     }
   }
}
