#include <jni.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "jargon_Jlcm.h"
#include "lcm.h"

/*--------------------------------*/
/* Java-LCM bindings              */
/* author: A. Hebert (04/03/2002) */
/*--------------------------------*/

/*
Copyright (C) 2002 Ecole Polytechnique de Montreal

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
*/

static char nomstatic[132];
#include "except.h"         /* (try,catch,throw) macros for exception handling */
jmp_buf exception_buffer;   /* environment buffer used by except.h             */
static int exception_status=0;  /* variable holding the error code (except.h)      */

/* Java exception handling function */
void jraise_exc(JNIEnv *env, char *s)
{
   jclass newExcCls;
   newExcCls = (*env)->FindClass(env, "java/lang/IllegalArgumentException");
   if (newExcCls == 0) {
     printf("Unable to find the new exception class, give up.\n");
     return;
   }
   (*env)->ThrowNew(env, newExcCls, s);
   return;
}
/* abort condition setting */
void xabort_c(char *msg){
  strcpy(nomstatic,msg);
  throw(LCMError);
}

/* Java LCM bindings */
JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmop
(JNIEnv *env, jobject this, jstring name, jint imp, jint medium, jint impx) {
   jclass cls;
   jfieldID fid;
   jlong lcm_object;
   jint ilong;
   const char *str = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",str);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"fid(addr)=0 (object name '%s')",str);
     jraise_exc(env, nomstatic);
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);

   try{
     lcmop_c((lcm**)&lcm_object, (char*)str, (long)imp, (long)medium, (long)impx);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, str); /* Release the object identification string */

   (*env)->SetLongField(env, this, fid, lcm_object);
}

JNIEXPORT int_32 JNICALL Java_jargon_Jlcm_lcmAccess (JNIEnv *env, jobject this) {
   lcm *lcm_object;
   char namlcm[13], nammy[13];
   jint ilong;
   int_32 empty, lcml, access;

   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (lcm *)(long)(*env)->GetLongField(env, this, fid);
   if (lcm_object == 0) return 0;
   try{
     lcminf_c((lcm**)&lcm_object, namlcm, nammy, &empty, (int_32*)&ilong, &lcml, &access);
     return access;
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcminf (JNIEnv *env, jobject this) {
   jlong lcm_object;
   char namlcm[13], nammy[13];
   jboolean jempty, jlcml;
   jint ilong;
   jstring jstr;
   int_32 empty, lcml, access;

   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcminf_c((lcm**)&lcm_object, namlcm, nammy, &empty, (int_32*)&ilong, &lcml, &access);
     jempty = (empty == 1);
     jlcml = (lcml == 1);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   fid = (*env)->GetFieldID(env, cls, "name", "Ljava/lang/String;");
   jstr = (*env)->NewStringUTF(env, namlcm);
   (*env)->SetObjectField(env, this, fid, jstr);
   fid = (*env)->GetFieldID(env, cls, "nammy", "Ljava/lang/String;");
   jstr = (*env)->NewStringUTF(env, nammy);
   (*env)->SetObjectField(env, this, fid, jstr);
   fid = (*env)->GetFieldID(env, cls, "empty", "Z");
   (*env)->SetBooleanField(env, this, fid, jempty);
   fid = (*env)->GetFieldID(env, cls, "ilong", "I");
   (*env)->SetIntField(env, this, fid, ilong);
}

JNIEXPORT jstring JNICALL Java_jargon_Jlcm_lcmnxt (JNIEnv *env, jobject this, jstring name) {
   jclass cls;
   jfieldID fid;
   jlong lcm_object;
   jint ilong;
   jstring jstr;
   char buf[13];

   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   strcpy(buf, (char*)namp);
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the character string */
   try{
     lcmnxt_c((lcm**)&lcm_object, buf);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   return (*env)->NewStringUTF(env, buf);
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmcl (JNIEnv *env, jobject this, jint iact) {
   jclass cls;
   jfieldID fid;
   jlong lcm_object;

   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmcl_c((lcm**)&lcm_object, (long)iact);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmlib (JNIEnv *env, jobject this) {
   jclass cls;
   jfieldID fid;
   jlong lcm_object;

   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmlib_c((lcm**)&lcm_object);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   fflush(stdout);
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmppd
(JNIEnv *env, jobject this, jstring name, jint ilen, jint itylcm, jobject object) {
   jclass cls;
   jfieldID fid;
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   const char *str;
   jboolean isCopy;

   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot put a LCM object into another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot put a LCM object into another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node '%s' has an invalid type (%d)",namp,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   
   try{
     if (isCopy) {
       /* pinning is not possible */
       lcmput_c((lcm**)&lcm_object, (char*)namp, ilen, itylcm, (int_32*)lcm_daughter);
     } else {
       /* pinning is possible */
       lcmppd_c((lcm**)&lcm_object, (char*)namp, ilen, itylcm, (int_32*)lcm_daughter);
       refpush((lcm**)&lcm_object, (int_32*)lcm_daughter);
     }
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmput
(JNIEnv *env, jobject this, jstring name, jint ilen, jint itylcm, jobject object) {
   jclass cls;
   jfieldID fid;
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   const char *str;
   jboolean isCopy, *lcm_boolean;

   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot put a LCM object into another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 3){
     int i;
     str = (*env)->GetStringUTFChars(env, object, 0);
     lcm_daughter = (jint *)setara_c(ilen);
     for (i=0;i<ilen-1;i++) strncpy (&((char*)lcm_daughter)[4*i], &str[4*i], 4);
     strncpy (&((char*)lcm_daughter)[4*(ilen-1)], "    ", 4);
     strncpy (&((char*)lcm_daughter)[4*(ilen-1)], &str[4*(ilen-1)], 1+(strlen(str)-1)%4);
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 5){
     int i;
     lcm_boolean = (*env)->GetBooleanArrayElements(env, object, &isCopy);
     lcm_daughter = (jint *)setara_c(ilen);
     for (i=0;i<ilen;i++) {
       if (lcm_boolean[i] == JNI_TRUE) lcm_daughter[i] = 1;
       else lcm_daughter[i] = 0;
     }
   } else if ((int)itylcm == 6){
     int i, j=0;
     jfloat *lcm_complex = (jfloat *)setara_c(ilen);
     for (i = 0; i < (*env)->GetArrayLength(env, object); ++i) {
       jobject obj = (*env)->GetObjectArrayElement(env, object, i);
       jclass cls = (*env)->GetObjectClass(env, obj);
       fid = (*env)->GetFieldID(env, cls, "re", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(re)=0");
         abort();
       }
       lcm_complex[j++] = (jfloat)(*env)->GetDoubleField(env, obj, fid);
       fid = (*env)->GetFieldID(env, cls, "im", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(im)=0");
         abort();
       }
       lcm_complex[j++] = (jfloat)(*env)->GetDoubleField(env, obj, fid);
     }
     lcm_daughter = (jint *)lcm_complex;
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot put a LCM object into another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node '%s' has an invalid type (%d)",namp,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   
   try{
     lcmput_c((lcm**)&lcm_object, (char*)namp, ilen, itylcm, (int_32*)lcm_daughter);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 3){
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseStringUTFChars(env, object, str);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   } else if ((int)itylcm == 5){
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseBooleanArrayElements(env, object,(jboolean*)lcm_boolean, 0);
   } else if ((int)itylcm == 6){
     rlsara_c((int_32*)lcm_daughter);
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
}

JNIEXPORT jintArray JNICALL Java_jargon_Jlcm_lcmlen (JNIEnv *env, jobject this, jstring name) {
   jclass cls;
   jfieldID fid;
   jint ilong;
   jlong lcm_object;
   jintArray info;
   jint *cinfo;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   info  = (*env)->NewIntArray(env, 2); /* jintArray of dimension 2 */
   cinfo = (*env)->GetIntArrayElements(env, info, 0);
   try{
     lcmlen_c((lcm**)&lcm_object, (char*)namp, (int_32*)&cinfo[0], (int_32*)&cinfo[1]);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseIntArrayElements(env, info, cinfo, 0);
   return info;
}

JNIEXPORT jintArray JNICALL Java_jargon_Jlcm_lcmlel (JNIEnv *env, jobject this, jint iset) {
   jint ilong;
   jlong lcm_object;
   jintArray info;
   jint *cinfo;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   info  = (*env)->NewIntArray(env, 2); /* jintArray of dimension 2 */
   cinfo = (*env)->GetIntArrayElements(env, info, 0);
   try{
     lcmlel_c((lcm**)&lcm_object, (int_32)iset, (int_32*)&cinfo[0], (int_32*)&cinfo[1]);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseIntArrayElements(env, info, cinfo, 0);
   return info;
}

JNIEXPORT jboolean JNICALL Java_jargon_Jlcm_lcmgpd (JNIEnv *env, jobject this, jstring name, jobject object) {
   jclass cls;
   jfieldID fid;
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   long ilen, itylcm;
   long *address;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmlen_c((lcm**)&lcm_object, (char*)namp, (int_32*)&ilen, (int_32*)&itylcm);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot get a LCM object from another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, 0));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, 0));
   } else if ((int)itylcm == 3){
     return JNI_FALSE;
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, 0));
   } else if ((int)itylcm == 5){
     return JNI_FALSE;
   } else if ((int)itylcm == 6){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, 0));
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot get a LCM object from another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node '%s' has an invalid type (%d)",namp,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   try{
     lcmgpd_c((lcm**)&lcm_object, (char*)namp, (int_32**)&address);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   } else if ((int)itylcm == 6){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
   if (address == (long*)lcm_daughter) {
      /* pinning is a success */
      return JNI_TRUE;
   } else {
      /* pinning failed: you should use lcmget */
      return JNI_FALSE;
   }
}

JNIEXPORT jobject JNICALL Java_jargon_Jlcm_lcmget (JNIEnv *env, jobject this, jstring name) {
   jclass cls;
   jfieldID fid;
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   long ilen, itylcm;
   jobject object;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmlen_c((lcm**)&lcm_object, (char*)namp, (int_32*)&ilen, (int_32*)&itylcm);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot get a LCM object from another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     object = (*env)->NewIntArray(env, ilen);
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, 0));
   } else if ((int)itylcm == 2){
     object = (*env)->NewFloatArray(env, ilen);
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, 0));
   } else if ((int)itylcm == 3){
     lcm_daughter = (jint *)setara_c(ilen+1);
   } else if ((int)itylcm == 4){
     object = (*env)->NewDoubleArray(env, ilen/2);
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, 0));
   } else if ((int)itylcm == 5){
     lcm_daughter = (jint *)setara_c(ilen);
   } else if ((int)itylcm == 6){
     lcm_daughter = (jint *)setara_c(ilen);
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot get a LCM object from another (%s).",namp);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node '%s' has an invalid type (%d)",namp,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   try{
     lcmget_c((lcm**)&lcm_object, (char*)namp, (int_32*)lcm_daughter);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 3){
     ((char*)lcm_daughter)[(int)(4*ilen)]='\0';
     object = (*env)->NewStringUTF(env, (char*)lcm_daughter);
     rlsara_c((int_32*)lcm_daughter);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   } else if ((int)itylcm == 5){
     int i;
     jboolean *lcm_boolean;
     object = (*env)->NewBooleanArray(env, ilen);
     lcm_boolean = (*env)->GetBooleanArrayElements(env, object, 0);
     for (i=0;i<ilen;i++) {
       if (lcm_daughter[i]) lcm_boolean[i] = JNI_TRUE;
       else lcm_boolean[i] = JNI_FALSE;
     }
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseBooleanArrayElements(env, object, lcm_boolean, 0);
   } else if ((int)itylcm == 6){
     int i, j=0;
     jdouble val;
     float *lcm_d = (float *)lcm_daughter;
     cls = (*env)->FindClass(env, "jargon/Complex");
     if (cls == 0) {
       jraise_exc(env, "Complex lookup failed");
       abort();
     }
     object = (*env)->NewObjectArray(env, ilen/2, cls, NULL);
     for (i = 0; i < ilen/2; ++i) {
       jobject obj = (*env)->AllocObject(env, cls);
       fid = (*env)->GetFieldID(env, cls, "re", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(re)=0");
         abort();
       }
       val = (jdouble)lcm_d[j++];
       (*env)->SetDoubleField(env, obj, fid, val);
       fid = (*env)->GetFieldID(env, cls, "im", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(im)=0");
         abort();
       }
       val = (jdouble)lcm_d[j++];
       (*env)->SetDoubleField(env, obj, fid, val);
       (*env)->SetObjectArrayElement(env, object, i, obj);
     }
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
   return object;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmdid (JNIEnv *env, jobject this, jstring name) {
   jclass cls;
   jfieldID fid;
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter=(long)lcmdid_c((lcm**)&lcm_object, (char*)namp);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
   return (jlong)lcm_daughter;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmlid (JNIEnv *env, jobject this, jstring name, jint klong) {
   jclass cls;
   jfieldID fid;
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter=(long)lcmlid_c((lcm**)&lcm_object, (char*)namp, (long)klong);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
   return (jlong)lcm_daughter;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmdil (JNIEnv *env, jobject this, jint iset) {
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter=(long)lcmdil_c((lcm**)&lcm_object, (long)iset);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   return (jlong)lcm_daughter;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmlil (JNIEnv *env, jobject this, jint iset, jint klong) {
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter=(long)lcmlil_c((lcm**)&lcm_object, (long)iset, (long)klong);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   return (jlong)lcm_daughter;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmgid (JNIEnv *env, jobject this, jstring name) {
   jclass cls;
   jfieldID fid;
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter = (jlong)(long)lcmgid_c((lcm**)&lcm_object, (char*)namp);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
   return (jlong)lcm_daughter;
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmgil (JNIEnv *env, jobject this, jint iset) {
   jint ilong;
   jlong lcm_object, lcm_daughter;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcm_daughter = (jlong)(long)lcmgil_c((lcm**)&lcm_object, (long)iset);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   return (jlong)lcm_daughter;
}

JNIEXPORT jboolean JNICALL Java_jargon_Jlcm_lcmgpl (JNIEnv *env, jobject this, jint iset, jobject object) {
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   long ilen, itylcm;
   long *address;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmlel_c((lcm**)&lcm_object, (long)iset, (int_32*)&ilen, (int_32*)&itylcm);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot get a LCM object from another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, 0));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, 0));
   } else if ((int)itylcm == 3){
     return JNI_FALSE;
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, 0));
   } else if ((int)itylcm == 5){
     return JNI_FALSE;
   } else if ((int)itylcm == 6){
     return JNI_FALSE;
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot get a LCM object from another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node %d has an invalid type (%d)",iset,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   try{
     lcmgpl_c((lcm**)&lcm_object, (long)iset, (int_32**)&address);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   }
   if (address == (long*)lcm_daughter) {
      /* pinning is a success */
      return JNI_TRUE;
   } else {
      /* pinning failed: you should use lcmget */
      return JNI_FALSE;
   }
}

JNIEXPORT jobject JNICALL Java_jargon_Jlcm_lcmgdl (JNIEnv *env, jobject this, jint iset) {
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   long ilen, itylcm;
   jobject object;
      
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmlel_c((lcm**)&lcm_object, (long)iset, (int_32*)&ilen, (int_32*)&itylcm);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot get a LCM object from another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     object = (*env)->NewIntArray(env, ilen);
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, 0));
   } else if ((int)itylcm == 2){
     object = (*env)->NewFloatArray(env, ilen);
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, 0));
   } else if ((int)itylcm == 3){
     lcm_daughter = (jint *)setara_c(ilen+1);
   } else if ((int)itylcm == 4){
     object = (*env)->NewDoubleArray(env, ilen/2);
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, 0));
   } else if ((int)itylcm == 5){
     lcm_daughter = (jint *)setara_c(ilen);
   } else if ((int)itylcm == 6){
     lcm_daughter = (jint *)setara_c(ilen);
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot get a LCM object from another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node %d has an invalid type (%d)",iset,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   try{
     lcmgdl_c((lcm**)&lcm_object, (int_32)iset, (int_32*)lcm_daughter);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 3){
     ((char*)lcm_daughter)[4*ilen]='\0';
     object = (*env)->NewStringUTF(env, (char*)lcm_daughter);
     rlsara_c((int_32*)lcm_daughter);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   } else if ((int)itylcm == 5){
     int i;
     jboolean *lcm_boolean;
     object = (*env)->NewBooleanArray(env, ilen);
     lcm_boolean = (*env)->GetBooleanArrayElements(env, object, 0);
     for (i=0;i<ilen;i++) {
       if (lcm_daughter[i]) lcm_boolean[i] = JNI_TRUE;
       else lcm_boolean[i] = JNI_FALSE;
     }
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseBooleanArrayElements(env, object, lcm_boolean, 0);
   } else if ((int)itylcm == 6){
     int i, j=0;
     jdouble val;
     float *lcm_d = (float *)lcm_daughter;
     cls = (*env)->FindClass(env, "jargon/Complex");
     if (cls == 0) {
       jraise_exc(env, "Complex lookup failed");
       abort();
     }
     object = (*env)->NewObjectArray(env, ilen/2, cls, NULL);
     for (i = 0; i < ilen/2; ++i) {
       jobject obj = (*env)->AllocObject(env, cls);
       fid = (*env)->GetFieldID(env, cls, "re", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(re)=0");
         abort();
       }
       val = (jdouble)lcm_d[j++];
       (*env)->SetDoubleField(env, obj, fid, val);
       fid = (*env)->GetFieldID(env, cls, "im", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(im)=0");
         abort();
       }
       val = (jdouble)lcm_d[j++];
       (*env)->SetDoubleField(env, obj, fid, val);
       (*env)->SetObjectArrayElement(env, object, i, obj);
     }
   }
   return object;
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmppl
(JNIEnv *env, jobject this, jint iset, jint ilen, jint itylcm, jobject object) {
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   const char *str;
   jboolean isCopy;

   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot put a LCM object into another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot put a LCM object into another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node %d has an invalid type (%d)",iset,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   try{
     if (isCopy) {
       /* pinning is not possible */
       lcmpdl_c((lcm**)&lcm_object, (int_32)iset, ilen, itylcm, (int_32*)lcm_daughter);
     } else {
       /* pinning is possible */
       lcmppl_c((lcm**)&lcm_object, (int_32)iset, ilen, itylcm, (int_32*)lcm_daughter);
       refpush((lcm**)&lcm_object, (int_32*)lcm_daughter);
     }
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   }
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmpdl
(JNIEnv *env, jobject this, jint iset, jint ilen, jint itylcm, jobject object) {
   jint ilong, *lcm_daughter;
   jlong lcm_object;
   const char *str;
   jboolean isCopy, *lcm_boolean;

   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   if ((int)itylcm == 0) {
     sprintf(nomstatic,"cannot put a LCM object into another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else if ((int)itylcm == 1){
     lcm_daughter = (jint *)((*env)->GetIntArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 2){
     lcm_daughter = (jint *)((*env)->GetFloatArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 3){
     int i;
     str = (*env)->GetStringUTFChars(env, object, 0);
     lcm_daughter = (jint *)setara_c(ilen);
     for (i=0;i<ilen-1;i++) strncpy (&((char*)lcm_daughter)[4*i], &str[4*i], 4);
     strncpy (&((char*)lcm_daughter)[4*(ilen-1)], "    ", 4);
     strncpy (&((char*)lcm_daughter)[4*(ilen-1)], &str[4*(ilen-1)], 1+(strlen(str)-1)%4);
   } else if ((int)itylcm == 4){
     lcm_daughter = (jint *)((*env)->GetDoubleArrayElements(env, object, &isCopy));
   } else if ((int)itylcm == 5){
     int i;
     lcm_boolean = (*env)->GetBooleanArrayElements(env, object, &isCopy);
     lcm_daughter = (jint *)setara_c(ilen);
     for (i=0;i<ilen;i++) {
       if (lcm_boolean[i] == JNI_TRUE) lcm_daughter[i] = 1;
       else lcm_daughter[i] = 0;
     }
   } else if ((int)itylcm == 6){
     int i, j=0;
     jfloat *lcm_complex = (jfloat *)setara_c(ilen);
     for (i = 0; i < (*env)->GetArrayLength(env, object); ++i) {
       jobject obj = (*env)->GetObjectArrayElement(env, object, i);
       jclass cls = (*env)->GetObjectClass(env, obj);
       fid = (*env)->GetFieldID(env, cls, "re", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(re)=0");
         abort();
       }
       lcm_complex[j++] = (jfloat)(*env)->GetDoubleField(env, obj, fid);
       fid = (*env)->GetFieldID(env, cls, "im", "D");
       if (fid == 0) {
         jraise_exc(env, "fid(im)=0");
         abort();
       }
       lcm_complex[j++] = (jfloat)(*env)->GetDoubleField(env, obj, fid);
     }
     lcm_daughter = (jint *)lcm_complex;
   } else if ((int)itylcm == 10){
     sprintf(nomstatic,"cannot put a LCM object into another (%d).",iset);
     jraise_exc(env, nomstatic);
     abort();
   } else {
     sprintf(nomstatic,"node %d has an invalid type (%d)",iset,itylcm);
     jraise_exc(env, nomstatic);
     abort();
   }
   
   try{
     lcmpdl_c((lcm**)&lcm_object, (int_32)iset, ilen, itylcm, (int_32*)lcm_daughter);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   if ((int)itylcm == 0) {
   } else if ((int)itylcm == 1){
     (*env)->ReleaseIntArrayElements(env, object,(jint*)lcm_daughter, 0);
   } else if ((int)itylcm == 2){
     (*env)->ReleaseFloatArrayElements(env, object,(jfloat*)lcm_daughter, 0);
   } else if ((int)itylcm == 3){
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseStringUTFChars(env, object, str);
   } else if ((int)itylcm == 4){
     (*env)->ReleaseDoubleArrayElements(env, object,(jdouble*)lcm_daughter, 0);
   } else if ((int)itylcm == 5){
     rlsara_c((int_32*)lcm_daughter);
     (*env)->ReleaseBooleanArrayElements(env, object,(jboolean*)lcm_boolean, 0);
   } else if ((int)itylcm == 6){
     rlsara_c((int_32*)lcm_daughter);
   }
}

JNIEXPORT jlong JNICALL Java_jargon_Jlcm_lcmequ (JNIEnv *env, jobject this, jstring name, jint imode) {
   jlong lcm_object1, lcm_object2;
   const char *nomob = (*env)->GetStringUTFChars(env, name, 0);
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object1 = (*env)->GetLongField(env, this, fid);
   try {
     lcmop_c((lcm**)&lcm_object2, (char*)nomob, 0, (long)imode, 0);
     lcmequ_c((lcm**)&lcm_object1, (lcm**)&lcm_object2);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, nomob); /* Release the object identification string */
   return (jlong)lcm_object2;
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmexp
(JNIEnv *env, jobject this, jint impx, jstring name, jint imode, jint idir) {
   jlong lcm_object;
   FILE *file;
   const char *nomob = (*env)->GetStringUTFChars(env, name, 0);
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   if (idir == 1) {
     file = fopen(nomob, "w");
   } else if (idir == 2) {
     file = fopen(nomob, "r");
   }
   try {
     lcmexp_c((lcm**)&lcm_object, (long)impx, file, (long)imode, (long)idir);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   fclose(file);
   (*env)->ReleaseStringUTFChars(env, name, nomob); /* Release the object identification string */
}

JNIEXPORT void JNICALL Java_jargon_Jlcm_lcmval (JNIEnv *env, jobject this,jstring name) {
   jclass cls;
   jfieldID fid;
   jint ilong;
   jlong lcm_object;
      
   const char *namp = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",namp);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     jraise_exc(env, "fid(addr)=0");
     abort();
   }
   lcm_object = (*env)->GetLongField(env, this, fid);
   try{
     lcmval_c((lcm**)&lcm_object, (char*)namp);
   } catch(LCMError) {
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, namp); /* Release the key string */
}
