#include <jni.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "jargon_Lifo.h"
#include "cle2000.h"

/*--------------------------------*/
/* Java-Lifo bindings             */
/* author: A. Hebert (14/08/2010) */
/*--------------------------------*/

/*
Copyright (C) 2010 Ecole Polytechnique de Montreal

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
*/

/* exception treatment. Note: These native method must be called from inside
  a try bloc in the Java side */
static char nomstatic[132];
#include "except.h"         /* (try,catch,throw) macros for exception handling */
jmp_buf exception_buffer;   /* environment buffer used by except.h             */
static int exception_status=0;  /* variable holding the error code (except.h)      */

/* Java Lifo bindings */
JNIEXPORT void JNICALL Java_jargon_Lifo_openNat(JNIEnv *env, jobject this) {
   jlong lifo_object;
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"openNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   cleopn((lifo**)&lifo_object);
   (*env)->SetLongField(env, this, fid, (jlong)lifo_object);
}
JNIEXPORT jint JNICALL Java_jargon_Lifo_closeNat(JNIEnv *env, jobject this) {
   jint ier;
   jlong lifo_object;
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"closeNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   ier = clecls((lifo**)&lifo_object);
   (*env)->SetLongField(env, this, fid, (jlong)lifo_object);
   return ier;
}
jobject jnodeNat(JNIEnv *env, lifo_node *my_node) {
   if (my_node->type < 0) {
     return NULL;
   } else if ((my_node->type == 3) || (my_node->type == 4)) {
     jlong lcm_object;
     jobject jnode;
     jclass nodecls;
     jfieldID nodefid;
     jmethodID nodeinit;

     lcm_object = (jlong)(my_node->value.mylcm);
     nodecls = (*env)->FindClass(env, "jargon/Jlcm");
     if (nodecls == 0) {
       jraise_exc(env, "Jlcm class lookup failed");
       abort();
     }
     nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "()V");
     if (nodeinit == 0) {
       jraise_exc(env, "Jlcm constructor lookup failed");
       abort();
     }
     jnode = (*env)->NewObject(env, nodecls, nodeinit);
     nodefid = (*env)->GetFieldID(env, nodecls, "addr", "J");
     if (nodefid == NULL) {
       sprintf(nomstatic,"jnodeNat: fid(addr)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     (*env)->SetLongField(env, jnode, nodefid, lcm_object);
     nodefid = (*env)->GetFieldID(env, nodecls, "type", "I");
     if (nodefid == NULL) {
       sprintf(nomstatic,"jnodeNat: fid(type)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     (*env)->SetIntField(env, jnode, nodefid, (my_node->type)-2);
     nodefid = (*env)->GetFieldID(env, nodecls, "name", "Ljava/lang/String;");
     if (nodefid == NULL) {
       sprintf(nomstatic,"jnodeNat: fid(type)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     (*env)->SetObjectField(env, jnode, nodefid, (*env)->NewStringUTF(env, my_node->OSname));
     return jnode;
   } else if ((my_node->type == 5) || (my_node->type == 6) || (my_node->type == 7)) {
     jclass nodecls = (*env)->FindClass(env, "java/io/File");
     if (nodecls == 0) {
       jraise_exc(env, "File class lookup failed");
       abort();
     }
     jmethodID nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "(Ljava/lang/String;)V");
     if (nodeinit == 0) {
       jraise_exc(env, "File constructor lookup failed");
       abort();
     }
     return (*env)->NewObject(env, nodecls, nodeinit, (*env)->NewStringUTF(env, my_node->OSname));
   } else if (my_node->type == 11) {
     jclass nodecls = (*env)->FindClass(env, "java/lang/Integer");
     if (nodecls == 0) {
       jraise_exc(env, "Integer class lookup failed");
       abort();
     }
     jmethodID nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "(I)V");
     if (nodeinit == 0) {
       jraise_exc(env, "Integer constructor lookup failed");
       abort();
     }
     return (*env)->NewObject(env, nodecls, nodeinit, my_node->value.ival);
   } else if (my_node->type == 12) {
     jclass nodecls = (*env)->FindClass(env, "java/lang/Float");
     if (nodecls == 0) {
       jraise_exc(env, "Float class lookup failed");
       abort();
     }
     jmethodID nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "(F)V");
     if (nodeinit == 0) {
       jraise_exc(env, "Float constructor lookup failed");
       abort();
     }
     return (*env)->NewObject(env, nodecls, nodeinit, my_node->value.fval);
   } else if (my_node->type == 13) {
     return (*env)->NewStringUTF(env, my_node->value.hval);
   } else if (my_node->type == 14) {
     jclass nodecls = (*env)->FindClass(env, "java/lang/Double");
     if (nodecls == 0) {
       jraise_exc(env, "Double class lookup failed");
       abort();
     }
     jmethodID nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "(D)V");
     if (nodeinit == 0) {
       jraise_exc(env, "Double constructor lookup failed");
       abort();
     }
     return (*env)->NewObject(env, nodecls, nodeinit, my_node->value.dval);
   } else if (my_node->type == 15) {
     jclass nodecls = (*env)->FindClass(env, "java/lang/Boolean");
     if (nodecls == 0) {
       jraise_exc(env, "Boolean class lookup failed");
       abort();
     }
     jmethodID nodeinit = (*env)->GetMethodID(env, nodecls, "<init>", "(Z)V");
     if (nodeinit == 0) {
       jraise_exc(env, "Boolean constructor lookup failed");
       abort();
     }
     return (*env)->NewObject(env, nodecls, nodeinit, (jboolean)my_node->value.ival);
   } else {
     sprintf(nomstatic,"jnodeNat: unknown node type (%d) for node %s", my_node->type, my_node->name);
     jraise_exc(env, nomstatic);
     abort();
   }
}
JNIEXPORT jobject JNICALL Java_jargon_Lifo_popNat(JNIEnv *env, jobject this) {
   lifo_node *my_node;
   jlong lifo_object;
   jobject jnode;
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"pop: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clepop((lifo**)&lifo_object);
   if (my_node == NULL) {
     sprintf(nomstatic,"pop: lifo stack pop failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   printf("LifoNat: pop node %s of type %d\n", my_node->name, my_node->type); fflush(stdout);
   jnode = jnodeNat(env, my_node);
   free(my_node);
   return jnode;
}
JNIEXPORT jobject JNICALL Java_jargon_Lifo_nodeNat(JNIEnv *env, jobject this, jstring jname) {
   lifo_node *my_node;
   jlong lifo_object;
   const char *node_name;
   jclass cls;
   jfieldID fid;
   jsize ilong;

   node_name = (*env)->GetStringUTFChars(env, jname, 0);
   ilong = (*env)->GetStringLength(env, jname);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",node_name);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"node: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clenode((lifo**)&lifo_object, (char *)node_name);
   if (my_node == NULL) {
     sprintf(nomstatic,"node: lifo stack node failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, jname, node_name); /* Release the character string */
   printf("LifoNat: find node %s of type %d\n", my_node->name, my_node->type); fflush(stdout);
   return jnodeNat(env, my_node);
}
JNIEXPORT jobject JNICALL Java_jargon_Lifo_posNat(JNIEnv *env, jobject this, jint ipos) {
   lifo_node *my_node;
   jlong lifo_object;
   jclass cls;
   jfieldID fid;

   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"pos: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clepos((lifo**)&lifo_object, ipos);
   if (my_node == NULL) {
     sprintf(nomstatic,"pos: lifo stack pos failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   printf("LifoNat: find node %s of type %d\n", my_node->name, my_node->type); fflush(stdout);
   return jnodeNat(env, my_node);
}
JNIEXPORT jstring JNICALL Java_jargon_Lifo_OSnodeNat(JNIEnv *env, jobject this, jstring jname) {
   lifo_node *my_node;
   jlong lifo_object;
   const char *node_name;
   jclass cls;
   jfieldID fid;
   jsize ilong;

   node_name = (*env)->GetStringUTFChars(env, jname, 0);
   ilong = (*env)->GetStringLength(env, jname);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",node_name);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"OSnodeNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clenode((lifo**)&lifo_object, (char *)node_name);
   if (my_node == NULL) {
     sprintf(nomstatic,"OSnodeNat: lifo stack node failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, jname, node_name); /* Release the character string */
   return (*env)->NewStringUTF(env, my_node->OSname);
}
JNIEXPORT jstring JNICALL Java_jargon_Lifo_OSposNat(JNIEnv *env, jobject this, jint ipos) {
   lifo_node *my_node;
   jlong lifo_object;
   jclass cls;
   jfieldID fid;

   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"OSposNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clepos((lifo**)&lifo_object, ipos);
   if (my_node == NULL) {
     sprintf(nomstatic,"OSposNat: lifo stack pos failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   return (*env)->NewStringUTF(env, my_node->OSname);
}
JNIEXPORT jint JNICALL Java_jargon_Lifo_typenodeNat(JNIEnv *env, jobject this, jstring jname) {
   lifo_node *my_node;
   jlong lifo_object;
   const char *node_name;
   jclass cls;
   jfieldID fid;
   jsize ilong;

   node_name = (*env)->GetStringUTFChars(env, jname, 0);
   ilong = (*env)->GetStringLength(env, jname);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",node_name);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"typeposNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clenode((lifo**)&lifo_object, (char *)node_name);
   if (my_node == NULL) {
     sprintf(nomstatic,"typeposNat: lifo stack node failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, jname, node_name); /* Release the character string */
   return my_node->type;
}
JNIEXPORT jint JNICALL Java_jargon_Lifo_typeposNat(JNIEnv *env, jobject this, jint ipos) {
   lifo_node *my_node;
   jlong lifo_object;
   jclass cls;
   jfieldID fid;

   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"OSposNat: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   my_node = clepos((lifo**)&lifo_object, ipos);
   if (my_node == NULL) {
     sprintf(nomstatic,"typeposNat: lifo stack pos failure");
     jraise_exc(env, nomstatic);
     abort();
   }
   return my_node->type;
}
JNIEXPORT void JNICALL Java_jargon_Lifo_pushNat(JNIEnv *env, jobject this, jstring name,
 jobject jnode, jstring stype, jstring josname) {
   jlong lifo_object, lcm_object;
   lifo_node *my_node;
   const char *node_name, *OS_name, *node_type;
   jclass cls, nodecls, classMethodAccess;
   jfieldID fid;
   jmethodID mid;
   jsize ilong;
   jstring jstr;

   node_name = (*env)->GetStringUTFChars(env, name, 0);
   ilong = (*env)->GetStringLength(env, name);
   if (ilong > 12) {
     sprintf(nomstatic,"object name '%s' limited to 12 characters",node_name);
     jraise_exc(env, nomstatic);
     abort();
   }
   cls = (*env)->GetObjectClass(env, this);
   fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == NULL) {
     sprintf(nomstatic,"push: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (*env)->GetLongField(env, this, fid);
   nodecls = (*env)->GetObjectClass(env, jnode);
   classMethodAccess = (*env)->FindClass(env, "java/lang/Class");
   mid = (*env)->GetMethodID(env, classMethodAccess, "getName", "()Ljava/lang/String;");
   if (mid == NULL) {
     sprintf(nomstatic,"push: mid(getName)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   jstr = (*env)->CallObjectMethod(env, nodecls, mid);
   node_type = (char *)(*env)->GetStringUTFChars(env, jstr, 0);
   printf("LifoNat: push node %s of type %s\n", node_name, node_type); fflush(stdout);
   my_node = (lifo_node *) malloc(sizeof(lifo_node));
   strcpy(my_node->name, node_name);
   if (strcmp(node_type,"jargon.Jlcm") == 0) {
     jint my_type, my_access;
     jboolean is_new;
     fid = (*env)->GetFieldID(env, nodecls, "addr", "J");
     if (fid == NULL) {
       sprintf(nomstatic,"push: fid(addr)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     lcm_object = (*env)->GetLongField(env, jnode, fid);
     my_node->value.mylcm = (lcm *)lcm_object;
     fid = (*env)->GetFieldID(env, nodecls, "name", "Ljava/lang/String;");
     if (fid == NULL) {
       jraise_exc(env, "push: fid(name)=0");
       abort();
     }
     jstr = (jstring)(*env)->GetObjectField(env, jnode, fid);
     OS_name = (*env)->GetStringUTFChars(env, jstr, 0);
     strcpy(my_node->OSname, OS_name);
     (*env)->ReleaseStringUTFChars(env, jstr, OS_name); /* Release the character string */
     fid = (*env)->GetFieldID(env, nodecls, "type", "I");
     if (fid == 0) {
       jraise_exc(env, "push: fid(type)=0");
       abort();
     }
     my_type = (jint)((*env)->GetIntField(env, jnode, fid));
     my_node->type = my_type + 2;
     fid = (*env)->GetFieldID(env, nodecls, "untoutched", "Z");
     if (fid == NULL) {
       sprintf(nomstatic,"push: fid(untoutched)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     is_new = ((*env)->GetBooleanField(env, jnode, fid) == JNI_TRUE);
     my_node->type = my_type + 2;
     fid = (*env)->GetFieldID(env, nodecls, "access", "I");
     if (fid == NULL) {
       sprintf(nomstatic,"push: fid(access)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     my_access = (*env)->GetIntField(env, jnode, fid);
     if (is_new) {
       my_node->access = 0;
     } else {
       my_node->access = my_access;
     }
   } else if (strcmp(node_type,"java.io.File") == 0) {
     const char *file_type;
     classMethodAccess = (*env)->FindClass(env, "java/io/File");
     mid = (*env)->GetMethodID(env, classMethodAccess, "getName", "()Ljava/lang/String;");
     if (mid == NULL) {
       sprintf(nomstatic,"push: mid(getName)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     jstr = (*env)->CallObjectMethod(env, jnode, mid);
     OS_name = (char *)(*env)->GetStringUTFChars(env, jstr, 0);
     strcpy(my_node->OSname, OS_name);
     (*env)->ReleaseStringUTFChars(env, jstr, OS_name); /* Release the character string */
     file_type = (char *)(*env)->GetStringUTFChars(env, stype, 0);
     if (strcmp(file_type, "BINARY") == 0) {
       my_node->type = 5;
     } else if (strcmp(file_type, "ASCII") == 0) {
       my_node->type = 6;
     } else if (strcmp(file_type, "DA") == 0) {
       my_node->type = 7;
     } else {
       sprintf(nomstatic,"push: unknown file type (%s) for node %s", file_type, node_name);
       jraise_exc(env, nomstatic);
     }
     (*env)->ReleaseStringUTFChars(env, stype, file_type); /* Release the character string */
   } else if (strcmp(node_type,"java.lang.Integer") == 0) {
     int my_integer;
     classMethodAccess = (*env)->FindClass(env, "java/lang/Integer");
     mid = (*env)->GetMethodID(env, classMethodAccess, "intValue", "()I");
     if (mid == NULL) {
       sprintf(nomstatic,"push: mid(intValue)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     my_integer = (*env)->CallIntMethod(env, jnode, mid);
     my_node->type = 11;
     my_node->value.ival = my_integer;
   } else if (strcmp(node_type,"java.lang.Float") == 0) {
     float my_float;
     classMethodAccess = (*env)->FindClass(env, "java/lang/Float");
     mid = (*env)->GetMethodID(env, classMethodAccess, "floatValue", "()F");
     if (mid == NULL) {
       sprintf(nomstatic,"push: mid(floatValue)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     my_float = (*env)->CallFloatMethod(env, jnode, mid);
     my_node->type = 12;
     my_node->value.fval = my_float;
   } else if (strcmp(node_type,"java.lang.String") == 0) {
     const char *my_string = (*env)->GetStringUTFChars(env, jnode, 0);
     my_node->type = 13;
     strcpy(my_node->value.hval, my_string);
     (*env)->ReleaseStringUTFChars(env, jnode, my_string); /* Release the character string */
   } else if (strcmp(node_type,"java.lang.Double") == 0) {
     double my_double;
     classMethodAccess = (*env)->FindClass(env, "java/lang/Double");
     mid = (*env)->GetMethodID(env, classMethodAccess, "doubleValue", "()D");
     if (mid == NULL) {
       sprintf(nomstatic,"push: mid(doubleValue)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     my_double = (*env)->CallDoubleMethod(env, jnode, mid);
     my_node->type = 14;
     my_node->value.dval = my_double;
   } else if (strcmp(node_type,"java.lang.Boolean") == 0) {
     jboolean my_boolean;
     classMethodAccess = (*env)->FindClass(env, "java/lang/Boolean");
     mid = (*env)->GetMethodID(env, classMethodAccess, "booleanValue", "()Z");
     if (mid == NULL) {
       sprintf(nomstatic,"push: mid(booleanValue)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     my_boolean = (*env)->CallBooleanMethod(env, jnode, mid);
     my_node->type = 15;
     my_node->value.ival = (int_32)my_boolean;
   } else if (strcmp(node_type,"java.lang.Class") == 0) {
     if (josname != NULL) {
       OS_name = (*env)->GetStringUTFChars(env, josname, 0);
       strcpy(my_node->OSname, OS_name);
       (*env)->ReleaseStringUTFChars(env, josname, OS_name); /* Release the character string */
     }
     jstr = (*env)->CallObjectMethod(env, jnode, mid);
     node_type = (char *)(*env)->GetStringUTFChars(env, jstr, 0);
     if (strcmp(node_type,"jargon.Jlcm") == 0) {
       const char *file_type = (char *)(*env)->GetStringUTFChars(env, stype, 0);
       if (strcmp(file_type, "LCM") == 0) {
         my_node->type = -3;
       } else if (strcmp(file_type, "XSM") == 0) {
         my_node->type = -4;
       } else {
         sprintf(nomstatic,"push: unknown LCM class type (%s) for node %s", file_type, node_name);
         jraise_exc(env, nomstatic);
       }
       (*env)->ReleaseStringUTFChars(env, stype, file_type); /* Release the character string */
     } else if (strcmp(node_type,"java.io.File") == 0) {
       const char *file_type = (char *)(*env)->GetStringUTFChars(env, stype, 0);
       if (strcmp(file_type, "BINARY") == 0) {
         my_node->type = -5;
       } else if (strcmp(file_type, "ASCII") == 0) {
         my_node->type = -6;
       } else if (strcmp(file_type, "DA") == 0) {
         my_node->type = -7;
       } else {
         sprintf(nomstatic,"push: unknown file class type (%s) for node %s", file_type, node_name);
         jraise_exc(env, nomstatic);
       }
       (*env)->ReleaseStringUTFChars(env, stype, file_type); /* Release the character string */
     } else if (strcmp(node_type,"java.lang.Integer") == 0) {
       my_node->type = -11;
     } else if (strcmp(node_type,"java.lang.Float") == 0) {
       my_node->type = -12;
     } else if (strcmp(node_type,"java.lang.String") == 0) {
       my_node->type = -13;
     } else if (strcmp(node_type,"java.lang.Double") == 0) {
       my_node->type = -14;
     } else if (strcmp(node_type,"java.lang.Boolean") == 0) {
       my_node->type = -15;
     } else {
       sprintf(nomstatic,"push: unknown node class type (%s) for node %s", node_type, node_name);
       jraise_exc(env, nomstatic);
       abort();
     }
   } else {
     sprintf(nomstatic,"push: unknown node type (%s) for node %s", node_type, node_name);
     jraise_exc(env, nomstatic);
     abort();
   }
   (*env)->ReleaseStringUTFChars(env, name, node_name); /* Release the character string */
   (*env)->ReleaseStringUTFChars(env, jstr, node_type); /* Release the character string */
   clepush((lifo**)&lifo_object, my_node);
 }
JNIEXPORT void JNICALL Java_jargon_Lifo_libNat(JNIEnv *env, jobject this) {
   jlong lifo_object;
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"lib: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (jlong)((*env)->GetLongField(env, this, fid));
   clelib((lifo**)&lifo_object);
   fflush(stdout);
}
JNIEXPORT int_32 JNICALL Java_jargon_Lifo_maxNat(JNIEnv *env, jobject this) {
   lifo *lifo_object;
   jclass cls = (*env)->GetObjectClass(env, this);
   jfieldID fid = (*env)->GetFieldID(env, cls, "addr", "J");
   if (fid == 0) {
     sprintf(nomstatic,"lib: fid(addr)=0");
     jraise_exc(env, nomstatic);
     abort();
   }
   lifo_object = (lifo *)(long)((*env)->GetLongField(env, this, fid));
   return lifo_object->nup;
}
