#include <jni.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "jargon_Operator.h"
#include "cle2000.h"
#define maxent 64  /* maximum number of module arguments */

/*--------------------------------*/
/* Java-Dragon (Jargon) binding   */
/* author: A. Hebert (14/03/2001) */
/*--------------------------------*/

static char nomstatic[132];
#include "except.h"         /* (try,catch,throw) macros for exception handling */
jmp_buf exception_buffer;   /* environment buffer used by except.h             */
static int exception_status=0;  /* variable holding the error code (except.h)      */

/* Java exception handling function */
void jraise_exc(JNIEnv *env, char *s)
{
   jclass newExcCls;
   newExcCls = (*env)->FindClass(env, "java/lang/IllegalArgumentException");
   if (newExcCls == 0) {
     /* Unable to find the new exception class, give up. */
     printf("Unable to find the new exception class, give up.\n");
     return;
   }
   (*env)->ThrowNew(env, newExcCls, s);
   return;
}
/* abort condition setting */
void xabort_c(char *msg){
  strcpy(nomstatic,msg);
  throw(JargonError);
}

/* Java Dragon binding */
JNIEXPORT void JNICALL Java_jargon_Operator_execNat
   (JNIEnv *env, jobject this, jstring jname, jstring jcommand,
   jobjectArray jkentry) {
   
   int_32 i, ier;
   int_32 nentry,ientry[maxent],jentry[maxent];
   long long kentry[maxent];
   char hentry[maxent][13], hparam[maxent][73];
   const char *name = (*env)->GetStringUTFChars(env, jname, 0);
   const char *command = (*env)->GetStringUTFChars(env, jcommand, 0);
   jint n;
   FILE *filein;
   int_32 dramod();

   if (jkentry == 0) {
     n = 0;
   } else {
     n = (*env)->GetArrayLength(env, jkentry);
   }
   for (i = 0; i < n; ++i) {
     jstring jstr;
     const char *str;
     jint iact;
     jint iuntout;
     jfieldID fid;
     jobject obj = (*env)->GetObjectArrayElement(env, jkentry, i);
     jclass cls = (*env)->GetObjectClass(env, obj);

     fid = (*env)->GetFieldID(env, cls, "name", "Ljava/lang/String;");
     if (fid == 0) {
       sprintf(nomstatic,"ModuleNat: fid(name)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     jstr = (*env)->GetObjectField(env, obj, fid);
     str = (*env)->GetStringUTFChars(env, jstr, 0);
     strfil_c(hentry[i],(char *)str,12);
     strfil_c(hparam[i],(char *)str,72);

     fid = (*env)->GetFieldID(env, cls, "objtype", "I");
     if (fid == 0) {
       sprintf(nomstatic,"ModuleNat: fid(objtype)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     ientry[i] = (int_32)(*env)->GetIntField(env, obj, fid);

     fid = (*env)->GetFieldID(env, cls, "addr", "J");
     if (fid == 0) {
       sprintf(nomstatic,"ModuleNat: fid(addr)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     kentry[i] =(long long)(*env)->GetLongField(env, obj, fid);

     fid = (*env)->GetFieldID(env, cls, "access", "I");
     if (fid == 0) {
       sprintf(nomstatic,"ModuleNat: fid(access)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     iact = (jint)(*env)->GetIntField(env, obj, fid);
     if (iact == 0) {
       sprintf(nomstatic,"object '%s' is closed",str);
       jraise_exc(env, nomstatic);
       abort();
     }
     fid = (*env)->GetFieldID(env, cls, "untoutched", "Z");
     if (fid == 0) {
       sprintf(nomstatic,"ModuleNat: fid(untoutched)=0");
       jraise_exc(env, nomstatic);
       abort();
     }
     iuntout = (jint)((*env)->GetBooleanField(env, obj, fid));
     if (iuntout == JNI_TRUE) {
       jentry[i] = 0;
     } else {
       jentry[i] = (int_32)iact;
     }
     (*env)->ReleaseStringUTFChars(env, jstr, str); /* Release the name string */
   }

/* create a file containing the module data */
   filein = fopen(name, "r");
   if (filein == NULL) {
      sprintf(nomstatic,"redget file open failure: %s", name);
      jraise_exc(env, nomstatic);
      abort();
   }
   
/* execute a module */
   try{
     ier = clemod_c((char*)command, filein, n, hentry, ientry, jentry, (lcm **)kentry,
     hparam, &dramod);
   } catch(JargonError){
     jraise_exc(env, nomstatic);
     fflush(stdout);
     abort();
   }
   if (ier != 0) {
      sprintf(nomstatic,"clemod failure (%s). ier=%d", command, ier);
      jraise_exc(env, nomstatic);
      abort();
   }
   ier = fclose(filein);
   if (ier != 0) {
      sprintf(nomstatic,"redget file close failure: %s", name);
      jraise_exc(env, nomstatic);
      abort();
   }
   ier = remove(name);
   if (ier != 0) {
      sprintf(nomstatic,"redget file remove failure: %s", name);
      jraise_exc(env, nomstatic);
      abort();
   }
   fflush(stdout);
          
   (*env)->ReleaseStringUTFChars(env, jcommand, command); /* Release the command string */
   (*env)->ReleaseStringUTFChars(env, jname, name); /* Release the name string */
}

JNIEXPORT void JNICALL Java_jargon_Operator_exitNat(JNIEnv *env, jobject this) {
   exit(1);
}
