package jargon;

import java.util.*;
/**
 * This is an asynchroneous class intented to be subclassed into a user-defined
 * procedure written in Java or in CLE-2000. All user-defined procedures have
 * the capability to be executed on a remote server using the Remote Method
 * Invocation (RMI) mechanism of Java.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Procedure extends Observable implements ProcedureIntf, Runnable {
  private static final long serialVersionUID = 80L;
  static {
    System.out.println("Procedure class loaded");
  }
  /** Thread object used to run the procedure.
   */
  public Thread thread;
  /** Operator object created by the procedure.
   */
  public Operator operatorOut;
  /** Send message "run completed" to Observers.
   */
  public void notifyObserver() {
    setChanged();
    notifyObservers("run completed");
  }
  /** Implements the Runnable interface. Call the exec() method of the
   * Procedure subclass (or the dummy exec() method of Procedure).
   * @throws RuntimeException If a JargonException is thrown during execution
   * of the exec() method
   */
  public void run() {
    try { 
      this.operatorOut = this.exec();
    } catch(JargonException ex) {
      throw new RuntimeException(ex.toString());
    }
    Runtime.getRuntime().gc();
    this.notifyObserver();
  }
  /** Start a thread named "Procedure Thread" to execute the procedure.
   */
  public void startThread() {
    this.thread = new Thread(this, "Procedure Thread");
    this.thread.start();
  }
  /** Return the state of the execution thread.
   * @return boolean equal to true if the procedure is executing
   */
  public boolean isAlive() {
    if (this.operatorOut == null) {
      return true;
    } else if (this.thread == null) {
      return false;
    } else {
      return this.thread.isAlive();
    }
  }
  /** Wait for the procedure to finish its execution.
   */
  public void join() throws InterruptedException {
    while (this.operatorOut == null) Thread.sleep(1000);
    this.thread.join();
  }
  /** Default exec() method for debugging purposes. This class is
   * public as required by the Remote Method Invocation (RMI)
   * mechanism but should not be called directly. Use the <tt>run()</tt>
   * method instead.
   * @return Operator object embedding the output LCM object
   */
  public Operator exec() throws JargonException {
    return new Dummy();
  }
}
