package jargon;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.event.*;
import com.enterprisedt.net.ftp.*;
import java.io.*;
import java.util.*;
import java.rmi.*;
import org.apache.commons.net.bsd.RExecClient;

/** This class is a JFC/Swing based window for managing a ThreadPool object.
 * A graphical user interface permits to select, start and interrupt the
 * servers used in the embedded ThreadPool object. These operations are
 * performed using client ftp and telnet protocols. A ServerFrame object
 * may be used within a JargonFrame object.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class ServerFrame extends JFrame implements ActionListener, Observer {
  private static final long serialVersionUID = 500L;
  static {
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new RMISecurityManager());
    }
  }
  private JTextField ipField, userField, pwField;
  private JLabel status = new JLabel(" Enter a server identification");
  private JTable table = new JTable(new MyTableModel());
  private ThreadPool pool = new ThreadPool();
  private ThreadRSH[] myThreadRSH = null;
  /** 
   * Use this constructor to create a ServerFrame object.
   */
  ServerFrame() {
    setTitle("Server selection");
    setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
    JPanel serverPane = new JPanel();
    serverPane.setLayout(new BoxLayout(serverPane, BoxLayout.Y_AXIS));
    
    JPanel setPane = new JPanel();
    setPane.setLayout(new GridLayout(0, 2, 5, 5));
    setPane.add(ipField = new JTextField(15));
    setPane.add(new JLabel(" Set the IP identification"));
    setPane.add(userField = new JTextField(15));
    setPane.add(new JLabel(" Set the user account"));
    setPane.add(pwField = new JPasswordField(15));
    setPane.add(new JLabel(" Set the password"));
    setPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    serverPane.add(setPane);

    JButton button;
    Box box = Box.createHorizontalBox();
    JPanel addPane = new JPanel();
    addPane.add(button = new JButton("Add"));
    box.add(addPane);
    button.addActionListener(this);
    JPanel deletePane = new JPanel();
    deletePane.add(button = new JButton("Delete"));
    box.add(deletePane);
    button.addActionListener(this);
    JPanel startPane = new JPanel();
    startPane.add(button = new JButton("Start"));
    box.add(startPane);
    button.addActionListener(this);
    JPanel stopPane = new JPanel();
    stopPane.add(button = new JButton("Stop"));
    box.add(stopPane);
    button.addActionListener(this);
    serverPane.add(box);
    
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
      // Handle JTable events
      public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) return ;
        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
        if (! lsm.isSelectionEmpty()) {
          System.out.println("Selection event index: " + lsm.getMinSelectionIndex());
          String ip = (String)table.getValueAt(lsm.getMinSelectionIndex(), 0);
          ipField.setText(ip);
          userField.setText((String)table.getValueAt(lsm.getMinSelectionIndex(), 1));
          pwField.setText((String)((MyTableModel)table.getModel()).getPw(lsm.getMinSelectionIndex()));
          status.setText(" Set server: " + ip);
        }
      }
    });
    table.setPreferredScrollableViewportSize(new Dimension(400, 150));
    table.getColumnModel().getColumn(0).setPreferredWidth(200);
    table.getColumnModel().getColumn(2).setPreferredWidth(50);
    JScrollPane scrollPane = new JScrollPane(table);
    
    // Read the server data from .rhosts file
    FileReader reader = null ;
    File myRhosts = new File(System.getProperty("user.home") + "/.rhosts");
    if (myRhosts.exists()) {
      try {
        reader = new FileReader(myRhosts);
      } catch (Exception e) {
        status.setText("Exception: " + e);
      }
      BufferedReader bin = new BufferedReader(reader);
      try {
        String Sinput ;
        int i = 0;
        while ((Sinput = bin.readLine()) != null) {
          int i1 = Sinput.indexOf(" ");
          int i2 = Sinput.indexOf(" ", i1+1);
          table.setValueAt(Sinput.substring(0, i1), i, 0);
          table.setValueAt(Sinput.substring(i1+1, i2), i, 1);
          table.setValueAt(new Boolean(false), i, 2);
          ((MyTableModel)table.getModel()).setPw(Sinput.substring(i2+1), i);
          i++;
        }
      } catch (Exception e) {
        status.setText("Exception: " + e);
      }
    }
    
    serverPane.add(scrollPane);
    serverPane.add(Box.createVerticalStrut(20));

    Container content = this.getContentPane();
    content.add(serverPane, BorderLayout.CENTER);
    content.add(status, BorderLayout.SOUTH);

    this.pack();
  }
  /** return the ThreadPool object managed by this ServerFrame object.
   */
  public ThreadPool getPool() {
    return this.pool;
  }
  
  // Custom table model
  private class MyTableModel extends AbstractTableModel {
    private static final long serialVersionUID = 501L;
    private String[] columnNames = {"IP server name", "user name", "running"};
    private Vector<Object> data = new Vector<Object>();
    private Vector<Object> running = new Vector<Object>();
    private Vector<Object> pwList = new Vector<Object>();
    
    public int getColumnCount() {
      return columnNames.length;
    }
    public int getRowCount() {
      return data.size();
    }
    public String getColumnName(int col) {
      return columnNames[col];
    }
    public Object getValueAt(int row, int col) {
      if (col < 2) {
        return ((Object[])data.elementAt(row))[col];
      } else {
        return running.elementAt(row);
      }
    }
    public void setValueAt(Object value, int row, int col) {
      if (col < 2) {
        if (row < data.size()) {
          String[] str = (String[])data.elementAt(row);
          str[col] = (String) value;
          data.setElementAt(str, row);
          fireTableCellUpdated(row, col);
        } else {
          String[] str = new String[2];
          str[col] = (String) value;
          data.addElement(str);
          fireTableRowsInserted(row, row);
        }
      } else {
        if (row < running.size()) {
          running.setElementAt(value, row);
          fireTableCellUpdated(row, col);
        } else {
          running.addElement(value);
          fireTableRowsInserted(row, row);
        }
      }
    }
    public Object getPw(int row) {
      return pwList.elementAt(row);
    }
    public void setPw(Object value, int row) {
      if (row < pwList.size()) {
        pwList.setElementAt(value, row);
      } else {
        pwList.addElement(value);
      }
    }
    public Class getColumnClass(int c) {
      return getValueAt(0, c).getClass();
    }
    public void removeElementAt(int row) {
      data.removeElementAt(row);
      running.removeElementAt(row);
      pwList.removeElementAt(row);
      fireTableRowsDeleted(row, row);
    }
  }
  /** implement the Observer interface. Handle observable events
   */
  public void update(Observable obj, Object arg) {
    System.out.println("ServerFrame observer: " + (String)arg + " detected for " + ((ThreadRSH)obj).host);
    if (((String)arg).equals("running")) {
      int n = table.getRowCount();
      for (int i=0; i<n; i++) {
        String host = (String)table.getValueAt(i, 0);
        if (host.equals(((ThreadRSH)obj).host)) {
          pool.add(((ThreadRSH)obj).host);
          table.setValueAt(new Boolean(true), i, 2);
          break;
        }
      }
      boolean scan = true;
      for (int i=0; i<n; i++) scan = scan & ((Boolean)table.getValueAt(i, 2)).booleanValue();
      if (scan) status.setText(" All servers started");
      repaint();
    } else if (((String)arg).equals("end")) {
      int n = table.getRowCount();
      for (int i=0; i<n; i++) {
        String host = (String)table.getValueAt(i, 0);
        if (host.equals(((ThreadRSH)obj).host)) {
          table.setValueAt(new Boolean(false), i, 2);
          break;
        }
      }
      repaint();
    }
  }
  /** implement ActionListener interface. Handle button events.
   */
  public void actionPerformed(ActionEvent e) {
    String str = e.getActionCommand();
    if (str.equals("Add")) {
      String ip = ipField.getText();
      for (int i=0; i<table.getRowCount(); i++) {
        if (((String)table.getValueAt(i, 0)).equals(ip)) {
          status.setText(" Server " + ip + " already defined");
          return;
        }
      }
      int index = table.getRowCount();
      table.setValueAt(ip, index, 0);
      table.setValueAt(userField.getText(), index, 1);
      table.setValueAt(new Boolean(false), index, 2);
      ((MyTableModel)table.getModel()).setPw(pwField.getText(), index);
      status.setText(" Set server: " + ip);
    } else if (str.equals("Delete")) {
      String ip = ipField.getText();
      for (int i=0; i<table.getRowCount(); i++) {
        if (((String)table.getValueAt(i, 0)).equals(ip)) {
          ((MyTableModel)table.getModel()).removeElementAt(i);
          status.setText(" Server: " + ip + " deleted");
          return;
        }
      }
      status.setText(" Unable to find server: " + ip);
    } else if (str.equals("Start")) {
      int n = table.getRowCount();
      if (n == 0) {
        status.setText(" No server defined");
        return;
      }
      myThreadRSH = new ThreadRSH[n];
      for (int i=0; i<n; i++) {
        String host = (String)table.getValueAt(i, 0);
        String user = (String)table.getValueAt(i, 1);
        String pw = (String)((MyTableModel)table.getModel()).getPw(i);
        System.out.println("Try to start server " + host);
        myThreadRSH[i] = new ThreadRSH(host, user, pw, this);
      }
      status.setText(" Starting servers...");
    } else if (str.equals("Stop")) {
      int n = table.getRowCount();
      if (n == 0) {
        status.setText(" No server defined");
        repaint();
        return;
      }
      for (int i=0; i<n; i++) {
        if (((Boolean)table.getValueAt(i, 2)).booleanValue()) myThreadRSH[i].stop();
      }
      status.setText(" Stopping servers...");
    } else {
      status.setText(" Unknown action: " + str);
    }
    repaint();
  }
  
  private class ThreadRSH extends Observable implements Runnable, Observer {
    String host;
    String user;
    String pw;
    Process p = null;
    Thread t;
    boolean on;
    ThreadRSH (String host, String user, String pw, Observer frame) {
      t = new Thread(this, "RSH thread");
      this.host = host;
      this.user = user;
      this.pw = pw;
      this.addObserver(this);
      this.addObserver(frame);
      this.on = true;
      t.start();
    }
    public void stop() {
      Thread s = new Thread(this, "RSH thread killer");
      this.on = false;
      s.start();
    }
    public void update(Observable obj, Object arg) {
      System.out.println("Observer: " + (String)arg + " detected");
      if (((String)arg).equals("binded")) {
        // The server is binded; remains to test if it can accept RMI requests.
        Procedure Test_obj = new Procedure();
        String serverURL = "rmi://" + this.host + "/ProcServer";
        System.out.println("ThreadRSH: Send request to server=" + this.host);
        System.out.flush();
        try {
          ProcServerIntf procServerIntf = (ProcServerIntf)Naming.lookup(serverURL);
          System.out.println("ProcServer lookup in ThreadRSH");
          Test_obj.operatorOut = procServerIntf.executeProc(Test_obj);
          Test_obj.join();
        } catch(Exception ex) {
          System.out.println("Exception: " + ex);
        }
        if (((float[])Test_obj.operatorOut.lcmObj.get("key"))[0] == 1.0f) {
          setChanged();
          notifyObservers("running");
        }
      }
    }
    public void run() {
      String serverName = host.substring(0, host.indexOf('.'));
      String command = null;
      if (this.on) {
        //
        // Set a FTP client to send the jargon.jar file on the server
        try {
          FTPClient ftp = new FTPClient(host, 21);
          ftp.login(user, pw);
          ftp.chdir("/tmp");
          ftp.setType(FTPTransferType.BINARY);
          ftp.put("jargon.jar", "jargon.jar");
          ftp.quit();
        } catch(Exception ae) {
          status.setText(" FTP client failure on " + serverName + " " + ae);
          return;
        }
        System.out.println("FTP access completed on " + host);
        command = "`find ~ -name startServer -print` &";
      } else {
        command = "`find ~ -name stopServer -print`";
      }
      //
      // Set a rexec client to start/stop the registry on the server
      RExecClient client = new RExecClient();
      try {
        client.connect(this.host);
      } catch (IOException e) {
        e.printStackTrace();
        status.setText(" rexec client failure on " + serverName + " " + e);
      }
      try {
        client.rexec(this.user, this.pw, command);
      } catch (IOException e) {
        try {
          client.disconnect();
        } catch (IOException f) {}
        e.printStackTrace();
        status.setText(" Could not execute command on " + serverName + " " + e);
        return;
      }
      InputStream in = client.getInputStream();
      BufferedReader bin = new BufferedReader(new InputStreamReader(in));
      try {
        String Sinput ;
        while ((Sinput = bin.readLine()) != null) {
          if (Sinput.indexOf("ProcServer binded") >= 0) {
            setChanged();
            notifyObservers("binded");
          } else if (Sinput.indexOf("server already running") >= 0) {
            setChanged();
            notifyObservers("binded");
          }
          System.out.println(serverName + ": " + Sinput);
        }
      } catch (Exception e) {
        status.setText(" Exception: " + e);
      }
      if (!this.on) {
        setChanged();
        notifyObservers("end");
      }
      try {
        client.disconnect();
      } catch (IOException e) {
        status.setText(" Exception: " + e);
      }
    }
  }
}
