package jargon;
/**
 * The Trivaa operator produces a Jlcm object containing the multigroup
 * finite element mass and stiffness matrices required by the iterative
 * flux solution of Trivac.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Trivaa extends Operator {
  static {
    System.out.println("Trivaa class loaded");
  }
  static String fr = "calcul d'un objet systeme de type Trivac";
  static String en = "calculation of a Trivac-type system object";
  private static final long serialVersionUID = 70L;
  /** set to <tt>true</tt> to skip the system matrix assembly but
   * to perform the L-D-LT factorization. Use the system matrices
   * already present in the Trivaa object.
   */
  public boolean skip;
  /** perturbation calculation flag. This variable is chosen among the
   * following values:
   * <ul><dl>
   * <dt> <tt>"DERI"</tt> <dd> the information recovered in the existing
   * Trivaa object is used as derivatives of nuclear properties with respect
   * to a state variable. Derivatives of system matrices with respect to the
   * same state variable are computed.
   * <dt> <tt>"PERT"</tt> <dd> The information recovered from the existing
   * Trivaa object is used as the perturbation of the nuclear properties.
   * Perturbations of the system matrices are computed.
   * </dl></ul>
   */
  public String delta;
  /** user-selected option list. Choose one or many options among the
   * following values:
   * <ul><dl>
   * <dt> <tt>"UNIT"</tt> <dd> compute a system matrix corresponding to cross
   * sections all set to one.
   * <dt> <tt>"OVEL"</tt> <dd> the reciprocal neutron velocities for each
   * material mixture are recovered from the input Macrolib object and used
   * to compute the corresponding system matrices, as required by the improved
   * quasistatic (IQS) method. 
   * </dl></ul>
   */
  public String[] options;
  /**
   * Use this constructor to create a new Trivaa object.
   * @param name user-defined name of the Trivaa object
   */
  public Trivaa(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  /** create a new embedded LCM structure and perform the assembly of
   * the multigroup system matrices, as specified by the instance
   * variables of this object.
   * @param a Macrolib object containing the cross sections.
   * @param b Tracking object containing energy-independent information.
   */
  public void exec(Macrolib a, Tracking b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.exec();
  }  
  /** recover the embedded LCM structure in an existing Trivaa object
   * and perform the assembly of the multigroup system matrices, as
   * specified by the instance variables of this object.
   * @param a Existing Trivaa object.
   * @param b Macrolib object containing the cross sections.
   * @param c Tracking object containing energy-independent information.
   */
  public void exec(Trivaa a, Macrolib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.exec();
  }  
  protected void makeText() {
    this.operator="TRIVAA: ";
    super.write("EDIT", super.edit);
    if (this.skip)  super.write("\nSKIP");
    if (this.delta != null) {
      super.write("\n"); super.write(this.delta);
    }
    if (this.options != null) {
      super.write("\nPNOR"); super.write(this.options);
    }
    super.write(";");
  }
}
