package texmaker;

import com.sun.javadoc.*;
import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * Modify a Java code and HTML text into Latex.
 * @author Frederic Serin - IUT Le Havre
 * @author Gregg Wonderly - C2 Technologies Inc.
 * @version 1.1.2, June 26, 2000
 */
public class Java2Latex {

/**
 * Translate Java code to Latex.
 * <P>
 * Parse the specific symbols: _, $, &, #, and ~.
 */
public static String fromCode(String str) {
	StringBuffer ret=new StringBuffer(str.length());

// loop to search special symbols

	for(int i=0;i<str.length();++i){
	int c=str.charAt(i);
	switch(c) {
	case '_':
	case '$':
		ret.append('\\');
		ret.append((char)c);
		break;
	case '#':
		ret.append("\\#");
		break;
	case '~':
		ret.append("\\~{}");
		break;
	case '&':
if( str.length()>i+4 && str.substring(i,i+5).equalsIgnoreCase("&amp;") ) {
		ret.append("\\&");
		i+=4;
		} else 
		ret.append("\\&");
		break;
	default:
		ret.append((char)c);
		break;
	} // end of switch
	} // end of loop

	return (ret.toString());
}

/**
 * Translate HTML text to Latex.
 * <P>
 * <B>Since version 1.0</B> :<BR>
 * Accept the following HTML tags :
 * &lt;br&gt;, &lt;p&gt;, &lt;b&gt;, &lt;i&gt; 
 * (and  &lt;/b&gt;, &lt;/i&gt;).
 * <P>
 * <B>Since version 1.1</B> :<BR>
 * Titles : &lt;h3&gt; for section, &lt;h4&gt; for subsection,
 * &lt;h5&gt; for subsubsection, &lt;h6&gt; for paragraph.<BR>
 * Unordered lists : &lt;ul&gt;, &lt;li&gt;, and &lt;/ul&gt;.<BR>
 * Ordered lists (&lt;ol&gt;) are parsed as unordered lists (sic).<BR>
 * Also : &lt;code&gt; and &lt;/code&gt;, &agrave;, &acirc;, 
 * &ccedil;, &eacute;, &egrave;, &ecirc;, &ugrave;, &iuml;, and &copy;.
 * @see TexDoclet
 * @param str the HTML text.
 * @return the Latex text.
 */
public static String fromHTML(String str) {
	StringBuffer ret=new StringBuffer(str.length());

// loop to search special symbols

boolean isVerbatim = false;
for(int i=0;i<str.length();++i){
int c=str.charAt(i);
switch(c) {
case '^':
	ret.append("$\\wedge$");
	break;
case '{':
	if (isVerbatim) {
          ret.append("{");
        } else {
          ret.append("$\\{$");
        }
	break;
case '}':
	if (isVerbatim) {
          ret.append("}");
        } else {
          ret.append("$\\}$");
        }
	break;
case '<':
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<br>") ){
		ret.append("\\\\" );
		i+=3;
		} else 
if(str.length()>i+2 && str.substring(i,i+3).equalsIgnoreCase("<p>") ){
		ret.append(" \\newline ");
		i+=2;
		} else 
if(str.length()>i+2 && str.substring(i,i+3).equalsIgnoreCase("<b>") ){
		ret.append("{\\bf " );
		i+=2;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("</b>") ){
		ret.append("}" );
		i+=3;
		} else 
if(str.length()>i+2 && str.substring(i,i+3).equalsIgnoreCase("<i>") ){
		ret.append("{\\it " );
		i+=2;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("</i>") ){
		ret.append("}" );
		i+=3;
		} else 
if(str.length()>i+5 && str.substring(i,i+6).equalsIgnoreCase("<code>") ){
		ret.append("{\\tt " );
		i+=5;
		} else 
if(str.length()>i+6 && str.substring(i,i+7).equalsIgnoreCase("</code>") ){
		ret.append("}" );
		i+=6;
		} else
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<ul>") ){
		ret.append("\\par\\begin{itemize} " );
		i+=3;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<ol>") ){
		ret.append("\\par\\begin{itemize} " );
		i+=3;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<li>") ){
		ret.append("\\par\\item " );
		i+=3;
		} else 
if(str.length()>i+4 && str.substring(i,i+5).equalsIgnoreCase("</ul>") ){
		ret.append("\\par\\end{itemize}" );
		i+=4;
		} else 
if(str.length()>i+4 && str.substring(i,i+5).equalsIgnoreCase("</ol>") ){
		ret.append("\\par\\end{itemize}" );
		i+=4;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<h3>") ){
		ret.append("\\section{" );
		i+=3;
		} else
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<h4>") ){
		ret.append("\\subsection{" );
		i+=3;
		} else
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<h5>") ){
		ret.append("\\subsubsection{" );
		i+=3;
		} else
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("<h6>") ){
		ret.append("\\paragraph{" );
		i+=3;
		} else
if(str.length()>i+4 && str.substring(i,i+5).equalsIgnoreCase("<pre>") ){
		ret.append("\\par\\begin{verbatim}" );
                isVerbatim = true;
		i+=4;
		} else
if(str.length()>i+5 && str.substring(i,i+6).equalsIgnoreCase("</pre>") ){
		ret.append("\\end{verbatim}" );
                isVerbatim = false;
		i+=5;
		} else
if(str.length()>i+4 && str.substring(i,i+3).equalsIgnoreCase("</h") ){
		ret.append("}" );
		i+=4;
		} else
while (str.length()>i && (! str.substring(i,i+1).equalsIgnoreCase(">")))
	i+=1; // search ">" character
	break;
case '/':
	ret.append("/");
	break;
case 'L':
if( str.length()>i+4 && str.substring(i,i+5).equals("LaTeX") ) {
		ret.append("\\LaTeX{}");
		i+=4;
		break;
		} else
		{
		ret.append((char)c);
		break;
		}
case '&':
if( str.length()>i+5 && str.substring(i,i+6).equalsIgnoreCase("&copy;") ) {
		ret.append("\\copyright");
		i+=5;
		} else
if(str.length()>i+5 && str.substring(i,i+6).equalsIgnoreCase("&nbsp;")) {
		ret.append("{\\tt\\ }"); 	
		i+=5;
		} else 
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("&lt;") ) {
		ret.append("\\textless ");
                i += 3;
		} else
if(str.length()>i+3 && str.substring(i,i+4).equalsIgnoreCase("&gt;") ) {
		ret.append("\\textgreater ");
		i+=3;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&eacute;") ) {
		ret.append("\\'e");
		i+=7;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&egrave;") ) {
		ret.append("\\`e");
		i+=7;
		} else
if(str.length()>i+6 && str.substring(i,i+7).equals("&ecirc;") ) {
		ret.append("\\^e");
		i+=6;
		} else
if(str.length()>i+5 && str.substring(i,i+6).equals("&iuml;") ) {
		ret.append("\\\"{i}");
		i+=5;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&agrave;") ) {
		ret.append("\\`a");
		i+=7;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&ccedil;") ) {
		ret.append("\\c{c}");
		i+=7;
		} else
if(str.length()>i+6 && str.substring(i,i+7).equals("&acirc;") ) {
		ret.append("\\^a");
		i+=6;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&ugrave;") ) {
		ret.append("\\`u");
		i+=7;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&Eacute;") ) {
		ret.append("\\'E");
		i+=7;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&Egrave;") ) {
		ret.append("\\`E");
		i+=7;
		} else
if(str.length()>i+6 && str.substring(i,i+7).equals("&Ecirc;") ) {
		ret.append("\\^E");
		i+=6;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&Agrave;") ) {
		ret.append("\\`A");
		i+=7;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&Ccedil;") ) {
		ret.append("\\c{C}");
		i+=7;
		} else
if(str.length()>i+6 && str.substring(i,i+7).equals("&Acirc;") ) {
		ret.append("\\^A");
		i+=6;
		} else
if(str.length()>i+7 && str.substring(i,i+8).equals("&Ugrave;") ) {
		ret.append("\\`U");
		i+=7;
		} else
		ret.append("&");
	break;
case '\\':
	ret.append("\\bslash ");
	break;
default:
	ret.append((char)c);
	break;
}
}
return (ret.toString());
}

} // end of HTML2Latex class declaration
