package texmaker;

import com.sun.javadoc.*;
import java.io.*;
import java.util.*;
//import java.awt.*;

/**
 * TexDoclet is a doclet generating LaTex2e document 
 * call by javadoc instruction.
 * @author F. Serin
 * @version 1.3, June 1, 2000
 */
public class TexDoclet extends Doclet {
/**
 * Specify that authors' names appear in chapters.
 */
static boolean author=false;
/**
 * Indicate an author name for the cover.
 */
static public String docAuthor="";
/**
 * Indicate the title of the cover.
 */
static public String docTitle="User's Guide";
/**
 * Specify that version and since tags appear in chapters.
 */
static boolean version=false;
/**
 * Name of the Tex document (with .tex suffix)
 */
static private String outFile="jargon.tex";
/**
 * Identifier of the Tex document (without suffix .tex)
 */
static PrintWriter masterDoc;

/**
 * Name of the parameters document.
 */
static private String dataFile=null;

/**
 * Specify that the document has one part per package (true) 
 * or no part (false).
 */
static boolean partition=true;
/**
 * Specify if the date is indicated on the cover.
 * <P>
 * By defaulte date=false (no date).
 */
static private boolean date=false;
/**
 * Contain all the packages.
 */
static private Vector packages=new Vector();

/**
 * Check for doclet added options here.
 */
public static int optionLength(String option){
	if (option.equals("-author")) return 1;
	else
	if (option.equals("-date")) return 1;
	else
	if (option.equals("-docAuthor")) return 2;
	else
	if (option.equals("-docTitle")) return 2;
	else
	if (option.equals("-output")) return 2;
	else
	if (option.equals("-version")) return 1;
	else
	if (option.equals("-gui")) return 2;

	System.out.println("TexMaker: unknown option "+option);
	return Doclet.optionLength(option);
	}

/**
 * Check that options have the correct arguments here.
 * @return true if the options are valid
 */
public static boolean validOptions(String[][] options, DocErrorReporter err){
	for (int i=0; i<options.length; ++i){
	if (options[i][0].equals("-author"))
		author=true;
	else
	if (options[i][0].equals("-version"))
		version=true;
	else
	if (options[i][0].equals("-date"))
		date=true;
	else
	if (options[i][0].equals("-docAuthor"))
		docAuthor=options[i][1];
	else
	if (options[i][0].equals("-output"))
		outFile=options[i][1];
	else
	if (options[i][0].equals("-docTitle"))
		docTitle=options[i][1];
	else
	if (options[i][0].equals("-gui"))
		dataFile=options[i][1];
	}
	return true;
	} // end of validOptions method


/**
 * Generate documentation here.
 * @return true on success
 */
public static boolean start(RootDoc root){
	try {
	masterDoc=new PrintWriter(new FileWriter(outFile));
	} catch (Exception ex) {}

	if (dataFile!=null) readFromFile(dataFile);

	PackageDoc[] thePackages=root.specifiedPackages();

	if (thePackages.length==1){
		if (docTitle.equals("User's Guide"))
		docTitle=docTitle+" for the package "+thePackages[0].name();
		partition=false;
		}

	createCover();

	new XtraChapter(masterDoc,root,"Overview");

	for (int i=0; i<thePackages.length; ++i){
		packages.add(thePackages[i]);
		}

	TexPackage texP;
	if (partition) {
		for (int i=0; i<thePackages.length; ++i){
			texP=new TexPackage(thePackages[i]);
			print("\\input "+texP.getFileName());
			} // end of loop
		}
	else {
		texP=new TexPackage(thePackages);
		print("\\input "+texP.getFileName());
		}

	endOfTheDocument();
	return true;
	} // end of start method

/**
 * Create the first instructions for the Latex document.
 * <P>
 * Include the title, the authors and the date of the document.
 */
private static void createCover(){
	print("\\documentclass[11pt,letterpaper]{article}");

	//print("\\usepackage[dvips]{graphicx}");
	//print("\\pagestyle{headings}");
	print("\\parindent=0pt");
	print("\\setlength{\\textwidth}{6.5in}");
	print("\\setlength{\\oddsidemargin}{0pt}");
	print("\\setlength{\\evensidemargin}{0pt}");

	print("\\begin{document}");
	print("\\title{\\huge{"+docTitle+"}}");
	if (! docAuthor.equals(""))
	print("\\author{"+docAuthor+"}");

	if (date)
		print("\\date{\\today}");
	else
		print("\\date{}");

	print("\\maketitle");
	print("\\strut\\thispagestyle{empty}");
	print("\\vfill");
	print("\\pagebreak");

	print("\\setcounter{page}{1}");

	print("\\tableofcontents");
	} // end of createCover method

/**
 * Create the end instructions of the Latex document.
 */
private static void endOfTheDocument(){
	print("\\end{document}");
	if (masterDoc != null){
		try {masterDoc.close();}
		catch (Exception ex) {}
		}
	}

/**
 * Say if the package is included in this dovument.
 */
public static boolean contains(PackageDoc pkg){
	return (packages.contains(pkg));
	}

public static void readFromFile(String fileName){
	try {
	BufferedReader br;
	br=new BufferedReader(new InputStreamReader(new
FileInputStream(fileName)));

	// part 1: unused informations
 	br.readLine();

	String packageName=br.readLine();
	while (! "end".equals(packageName)){
		packageName=br.readLine();
		} // end of loop

	// part 2: usefull informations
	if ("0".equals(br.readLine())) author=false;
	else author=true;

	if ("0".equals(br.readLine())) version=false;
	else version=true;

	if ("0".equals(br.readLine())) date=false;
	else date=true;

	docAuthor=br.readLine();

	String titleDoc=br.readLine();
	docTitle="";	
	while (titleDoc!=null){
		if (titleDoc.equals(""))
			docTitle=docTitle+" \\vspace{5mm}";
		else
			docTitle=docTitle+titleDoc+" \\\\";
		titleDoc=br.readLine();
		} // end of loop and end of file
	br.close();
	} catch (Exception ex) {}
	} // end of readFromFile method

/**
 * This method simplifies the code writing.
 * <P>
 * Do masterDoc.println(s) with a blank line after.
 * @param s string to print in os document.
 * @since 1.0
 */
public static void print(String s){
	masterDoc.println(Java2Latex.fromCode(s));
	masterDoc.println("");
	} // end of print method

/**
 * Method to simplify the code writing.
 * <P>
 * Do masterDoc.println(s).
 * @param s string to print in os document.
 * @since 1.0
 */
public static void printTitle(int i,String s){
	print(theTitle(i)+s+"}");
	}

/**
 * Method to simplify the code writing.
 * <P>
 * Do masterDoc.println(s).
 * @param s string to print in os document.
 * @param lab label used to reference this title
 * @since 1.0
 */
public static void printTitle(int i,String s, String lab){
	print(theTitle(i)+"\\label{"+lab+"}"+s+"}");
	}

/**
 * Print a title in the document.
 * @param i level of the title. From 0 for <I>part</I> to 7 for
<I>subparagraph</I>.
 */
public static String theTitle(int i){
	switch(i) {
	case 1 :
		return ("\\section{");
	case 2 :
		return ("\\subsection{");
	case 3 :
		return ("\\subsubsection{");
	case 4 :
		return ("\\paragraph{");
	case 5 :
		return ("\\subparagraph{");
	default :
		return ("{\\bf ");
	} // end of switch
	} // end of theTitle method

} // end of TexMaker doclet declaration
