package texmaker;

import com.sun.javadoc.*;
import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * This class generates one part per package. 
 * @author F. Serin
 * @version 1.2, June 3, 2000
 */
public class TexPackage extends TexDoc {
/**
 * Name of the Tex file for this package.
 */
private String fileName;

TexPackage(PackageDoc pkg){
	fileName=pkg.name()+".tex";

	try {
		outFile=new PrintWriter(new FileWriter(fileName));
	} catch (Exception ex) {}

	StringBuffer packTitle=new StringBuffer("\\part[");
	packTitle.append(pkg.name());
	packTitle.append("]{The package\\\\ ");
	packTitle.append(pkg.name());
	packTitle.append(" \\label{");
	packTitle.append(pkg.name());
	packTitle.append("}}");
	print(packTitle.toString());

	createIntro(pkg);

	PackageDoc[] onePackage = new PackageDoc [1];
	onePackage[0]=pkg;
	createChapters(onePackage);
	closeDocument();
	} // end of constructor

TexPackage(PackageDoc[] pkg){
	fileName="unknown.tex";

	try {
		outFile=new PrintWriter(new FileWriter(fileName));
	} catch (Exception ex) {}

	if (pkg.length==1)
		createIntro(pkg[0]);

	createChapters(pkg);
	closeDocument();
	} // end of constructor

/**
 * Return the name of the Tex file  containing the classes.
 */
public String getFileName(){
	return (fileName);
	}

/**
 * Create one section to present the package.
 */
private void createIntro(PackageDoc pkg){
	if ((pkg.inlineTags()).length>0){
		StringBuffer chap=new StringBuffer("\\section[");
		chap.append("Introduction]{{\\small ");
		chap.append(pkg.name());
		chap.append("}\\\\Introduction}");
		print(chap.toString());
		printComment(pkg);
		}
	}

/**
 * Create one section per class in the dedicated 
 * {@link #outFile Tex file}.
 */
private void createChapters(PackageDoc[] pkg){
	Vector interfaces=new Vector();
	Vector classes=new Vector();

	for (int i=0; i<pkg.length; ++i){
		ClassDoc[] theClasses=pkg[i].allClasses();

		for (int j=0; j<theClasses.length; ++j){
		if (theClasses[j].isInterface())
			interfaces.add(theClasses[j]);
		else
		if (theClasses[j].isOrdinaryClass()){
			classes.add(theClasses[j]);
		} // end of ordinary class case
		} // end of the loop of j
	} // end of the loop of i

// sort of classes (alpha order)

	Collections.sort(interfaces);
	Collections.sort(classes);

// creation of one section per class

	ClassDoc cls;
	Enumeration enum=interfaces.elements();
	while (enum.hasMoreElements()){
		cls=(ClassDoc)enum.nextElement();
		new TexClass(cls,cls.containingPackage(),outFile);
		} // end of while

	enum=classes.elements();
	while (enum.hasMoreElements()){
		cls=(ClassDoc)enum.nextElement();
		new TexClass(cls,cls.containingPackage(),outFile);
		} // end of while



	} // end of createDocument method

/**
 * Close the created Tex file.
 */
private void closeDocument(){
	if (outFile != null){
		try {outFile.close();}
		catch (Exception ex) {}
		} // fi
	} // end of closeDocument method

} // end of TexPackage class declaration
